!
!  Functions for utilizing the more basic F90 interface
!
!	AUTHOR:
!	  D. Hinshaw (DH)
!

module DSSutil

  use DSSinter
  use Dal

  implicit none

  private

  type RangeT
     integer             :: lowerB
     real (kind=double)  :: lower
     real (kind=double)  :: upper
     integer             :: upperB
  end type RangeT

  public RangeT

  integer, public, parameter :: OPEN = 1
  integer, public, parameter :: CLOSED = 2
  integer, public, parameter :: UNDEF = 3
     
  public dssRanges

  interface dssRanges
     module procedure dssSubspaceRanges
     module procedure dssComponentRanges
     module procedure dssFilterRanges
  end interface

contains

  function dssSubspaceRanges( dss, axis )

    type(RangeT), dimension(:), pointer :: dssSubspaceRanges
    type(DssT), intent(in)              :: dss
    character(len=*), intent(in)        :: axis

    integer, dimension(:), pointer           :: lowerB, upperB
    real(kind=double), dimension(:), pointer :: lower, upper
    integer                                  :: nSize

    call getDssRanges(dss, axis, lowerB, lower, upper, upperB)

    nSize = size(upper)
    allocate( dssSubspaceRanges(0:nSize-1) )

    dssSubspaceRanges%lowerB = lowerB
    dssSubspaceRanges%upperB = upperB
    dssSubspaceRanges%lower = lower
    dssSubspaceRanges%upper = upper

    call releaseDssRanges(lowerB, lower, upper, upperB)
    
  end function dssSubspaceRanges


  function dssComponentRanges( dscomp, axis )

    type(RangeT), dimension(:), pointer :: dssComponentRanges
    type(DScompT), intent(in)           :: dscomp
    character(len=*), intent(in)        :: axis

    integer, dimension(:), pointer           :: lowerB, upperB
    real(kind=double), dimension(:), pointer :: lower, upper
    integer                                  :: nSize

    call getDssRanges(dscomp, axis, lowerB, lower, upper, upperB)

    nSize = size(upper)
    allocate( dssComponentRanges(0:nSize-1) )

    dssComponentRanges%lowerB = lowerB
    dssComponentRanges%upperB = upperB
    dssComponentRanges%lower = lower
    dssComponentRanges%upper = upper

    call releaseDssRanges(lowerB, lower, upper, upperB)
    
  end function dssComponentRanges


  function dssFilterRanges( dsfilter )

    type(RangeT), dimension(:), pointer :: dssFilterRanges
    type(DSfilterT), intent(in)         :: dsfilter

    integer, dimension(:), pointer           :: lowerB, upperB
    real(kind=double), dimension(:), pointer :: lower, upper
    integer                                  :: nSize

    call getDssRanges(dsfilter, lowerB, lower, upper, upperB)

    nSize = size(upper)
    allocate( dssFilterRanges(0:nSize-1) )

    dssFilterRanges%lowerB = lowerB
    dssFilterRanges%upperB = upperB
    dssFilterRanges%lower = lower
    dssFilterRanges%upper = upper

    call releaseDssRanges(lowerB, lower, upper, upperB)
    
  end function dssFilterRanges

end module DSSutil
