//---------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	testDssBuilder.cc: --- test harness for DSS builder
//
//	CHANGE HISTORY:
//		UL, 1999-11-16: 1.0.0: - first version
//
//	REFERENCES:
//		[1]: Documentation of selectlib package, U. Lammers
//
//	AUTHOR:
//		U. Lammers (UL), Tue Nov 16 09:56:55 MET 1999
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/task_devel/packages/dsslib/test/testDssBuilder.cc,v 1.2 1999/11/18 18:06:38 ulammers Exp $
//

#include <cassert>
#include <string>
#include "DssBuilder.h"
#include "Evaluator.h"
#include "dss.h"
#include "errstr.h"

int main(int argc, char **argv)
{
	if (argc != 2) {
		cerr<<"Usage: "<<argv[0]<<" <expr>\n";
		return 1;
	}

	string expr(argv[1]);
	try {
		DataSubspace dss = dssBuilder.dss(expr);

		string dssexpr(dss.selectExpression());
		cout<<"Selection expression corresponding to DSS constructed from `"
			<<expr<<"' is `"<<dssexpr<<"'"<<endl;

		unsigned nComps = dss.components();
		cout<<"The DSS has >>"<<nComps<<"<< components"<<endl;
		for (unsigned n=0; n<nComps; ++n)
			cout<<"Component "<<n<<" has >>"<<dss.component(n)->filters()
				<<"<< filter(s)\n";

		//	see if we get a valid selection expression back
		bool isValid = Evaluator::isValidExpression(dssexpr);
		cout<<"Expression is ... "<<(isValid ? "valid" : "invalid")<<endl;

		return !isValid;
	} catch(SelectException e) {
		errstr<<e.text()<<Sas::error("DssException");
	}

	return 0;
}
