program testInterface

  use Dal
  use DSS
  use DataFilter

  type(DataSetT)  :: eset
  type(BlockT)    :: eblock
  type(TableT)    :: etab
  type(DssT)      :: edss, edss2
  type(DScompT)   :: edscomp
  type(DSfilterT) :: edsfilter

  type(RangeT), dimension(:), pointer :: ranges, ranges0

  character(len=*), parameter :: express = "( RAWX in [0:200],(300:600) && " // &
    "gti(templist.fits:STDGTI,TIME) && region(templist.fits:REGION,X,Y)"     // &
    " && FLAG&4095==0 && region(templist.fits:REG105,X,Y) && TIME in "       // &
    "[0.00000000000000:10000.0000000000] ) || ( RAWX in [0:10000] && FLAG&4095==0 && region("  // &
    "templist.fits:REG105,X,Y) && RAWY in [0:200],(300:600) && mask("        // &
    "templist.fits:MASK,0,0,DETX,DETY) ) || ( mask(templist.fits:MASK"       // &
    ",0,0,DETX,DETY) )"

  character(len=255) :: maskSpec

  eset = dataSet("templist.fits", Modify)
  etab = table(eset, "EVENTS")
  eblock = block(etab)

! Read in the DSS

  edss = dataSubspace(eblock)

! Check the selectlib expression

  print *, trim(selectExpression(edss))

  if( trim(selectExpression(edss)) /= express )            &
      call error("SelectExpress", "Expected selection expression not produced")

! Check retrieval of ranges

  ranges => dssRanges(edss, "RAWX")
  ranges0 => dssRanges( dssComponent(edss, 0), "RAWX" )

  if(  ranges0(0)%lowerB /= ranges(0)%lowerB .or.          &
       ranges0(0)%lower  /= ranges(0)%lower  .or.          &
       ranges0(0)%upperB /= ranges(0)%upperB .or.          &
       ranges0(0)%upper  /= ranges(0)%upper  )             &
       call error("RangeRetrieve", "Failed range retrieval test")

! See if filter with a component gives the correct result
  
  call initFilter( selectExpression(dssComponent(edss, 1)) )
  if( doSelect(etab) /= 33 ) call error("Express1", "Failed test expression 1")

! Make sure we can delete stuff

  call dssDeleteComponent(edss, 2)
  call dssDeleteComponent(edss, 0)

  edscomp = dssComponent(edss, 0)

  call dssDeleteFilter(edscomp, 0)
  call dssDeleteFilter(edscomp, 1)

  call initFilter( selectExpression(edss) )  
  if( doSelect(etab) /= 1516 )  &
       call error("Express2", "Failed test expression 2")

! Test the routines for getting filter info

  if( .not. dssHasFilter(edscomp, "DET", MASKFILTERT) ) &
       call error("HasFitler", "Unable to find valid filter.")
  
  edsfilter = dssFilter(edscomp, "DET", MASKFILTERT)

  if( dssFilterType(edsfilter) /= MASKFILTERT ) &
       call error("FilterType", "Retrieved wrong filter type")

  if( dssFilterName(edsfilter) /= "DET" ) &
       call error("FilterName", "Retrieved wrong filter name")

  if( dssFilterUnits(edsfilter) /= "" ) &
       call error("FilterUnits", "Retrieved wrong filter units")

  if( dssFilterComps(edsfilter) /= "DETX,DETY" ) &
       call error("FilterComps", "Retrieved wrong filter components")

  maskSpec = dssFilterBlock(edsfilter)
  if( maskSpec /= "templist.fits:MASK" ) &
       call error("FilterBlock", "Retrieved wrong filter block")

! Clean up

  deallocate(ranges)
  deallocate(ranges0)
  call dssRelease(edss)

! Read the subspace specification back in, and make a new table with it.

  edss = dataSubspace(eblock)
  edss2 = dataSubspace(eblock)

  call dssDeleteComponent(edss, 2)
  call dssDeleteComponent(edss, 1)

  call dssDeleteFilter( dssComponent(edss, 0), 5 )
  call dssDeleteFilter( dssComponent(edss, 0), 4 )
  call dssDeleteFilter( dssComponent(edss, 0), 3 )

  call release(eset) 

  call copyDataSet("emlist.fits","example.fits")

  eset = dataSet("example.fits", Modify)
  etab = table(eset, "EVENTS")
  eblock = block(etab)

  call initFilter( selectExpression(edss) )
  print *, doFilter(etab)
  call dssWrite(edss, eblock)

  call release(eset)

  ! Try merging to subspaces

  call dssMerge(edss, edss2)

  call dssRelease(edss)
  call dssRelease(edss2)

! Open the temporary file, and delete the DSS from it.

  eset = dataSet("templist.fits", Modify)
  etab = table(eset, "EVENTS")
  eblock = block(etab)

  call dssDelete(eblock)

  call release(eset)

end program testInterface
